#--------------------------------------------------------------------------
#     This file is part of BKchem - a chemical drawing program
#     Copyright (C) 2002  Beda Kosata <kosatab@vscht.cz>

#     This program is free software; you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation; either version 2 of the License, or
#     (at your option) any later version.

#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.

#     Complete text of GNU GPL can be found in the file gpl.txt in the
#     main directory of the program

#--------------------------------------------------------------------------
#
# Last edited: $Date: 2002/10/18 17:54:13 $
#
#--------------------------------------------------------------------------

"""set of dialogs used by BKchem"""

import Tkinter
import tkFont
import Pmw
import misc
import data
import re
import widgets


## SCALE DIALOG

class scale_dialog:
  """dialog used to get ratio for scaling in percent"""
  def __init__( self, parent):
    self.dialog = Pmw.Dialog( parent,
                              buttons=(_('OK'), _('Cancel')),
                              defaultbutton=_('OK'),
                              title=_('Scale'),
                              command=self.done)
    # X RATIO
    self.entryx = Pmw.Counter( self.dialog.interior(),
                               labelpos = 'w',
                               label_text=_("Scale X (in %):"),
                               entryfield_value = 100,
                               entryfield_validate={ 'validator':'integer', 'min':1, 'max':1000},
                               entry_width = 5,
                               entryfield_modifiedcommand = self._scalex_changed,
                               increment = 10,
                               datatype = 'integer')
    self.entryx.pack(pady=10, anchor='w', padx=10)
    # Y RATIO
    self.entryy = Pmw.Counter( self.dialog.interior(),
                               labelpos = 'w',
                               label_text=_("Scale Y (in %):"),
                               entryfield_value = 100,
                               entryfield_validate={ 'validator':'integer', 'min':1, 'max':1000},
                               entry_width = 5,
                               entryfield_modifiedcommand = self._scaley_changed,
                               increment = 10,
                               datatype = 'integer')
    self.entryy.pack(pady=10, anchor='w', padx=10)

    self.preserve_ratio = Tkinter.IntVar()
    self.preserve_ratio_entry = Tkinter.Checkbutton( self.dialog.interior(),
                                                     text=_('Preserve aspect ratio?'),
                                                     variable = self.preserve_ratio,
                                                     command = self._preserve_ratio_changed)
    self.preserve_ratio_entry.pack()

##     self.preserve_center = Tkinter.IntVar()
##     self.preserve_center_entry = Tkinter.Checkbutton( self.dialog.interior(),
##                                                       text=_('Preserve centers of containers?'),
##                                                       variable = self.preserve_center)
##     self.preserve_center_entry.pack()

    self.dialog.activate()

  def done( self, button):
    """called on dialog exit"""
    if not button or button == _('Cancel'):
      self.result = None
    elif not (self.entryx.valid() and self.entryy.valid()) :
      self.result = ()
    else:
      self.result = (float( self.entryx.get()), float( self.entryy.get())) #, self.preserve_center.get())
      if self.preserve_ratio.get():
        # x is significant if ratio should be preserved
        self.result = (self.result[0], self.result[0])
    self.dialog.deactivate()

  def _scalex_changed( self):
    if self.preserve_ratio.get():
      if self.entryy.get() != self.entryx.get():
        self.entryy.setentry( self.entryx.get())

  def _scaley_changed( self):
    if self.preserve_ratio.get():
      if self.entryy.get() != self.entryx.get():
        self.entryx.setentry( self.entryy.get())

  def _preserve_ratio_changed( self):
    if self.preserve_ratio.get():
      if self.entryy.get() != self.entryx.get():
        if self.entryx.get() == '100':
          self.entryx.setentry( self.entryy.get())
        else:
          self.entryy.setentry( self.entryx.get())
    

## CONFIG DIALOG

class config_dialog:
  """items configuration"""
  def __init__( self, parent, items):
    self.items = items
    self.changes_made = 0
    self.parent = parent
    self.dialog = Pmw.Dialog( parent,
                              buttons=(_('OK'), _('Cancel')),
                              defaultbutton=_('OK'),
                              title=_('Configuration'),
                              command=self.done,
                              master='parent')
    #parent.bind_all( "<Button-1>", self.raise_me, add='+')
    self.pages = Pmw.NoteBook( self.dialog.interior())
    self.pages.pack( anchor='w', pady=0, padx=0, fill='both', expand=1)
    
    # create pages for different item types
    self.atom_page = None
    self.bond_page = None
    self.arrow_page = None
    self.text_page = None
    self.plus_page = None
    self.font_page = None
    self.common_page = None
    arrows = []
    for o in items:
      if o.object_type == 'point':
        items.remove( o)
        if o.arrow not in arrows:
          arrows.append( o.arrow)
    items += arrows
    types = misc.filter_unique( [o.object_type for o in items])

    if 'atom' in types:
      self.atom_page = self.pages.add(_('Atom'))
      # charge
      charges = misc.filter_unique( [o.charge for o in items if o.object_type == 'atom'])
      if len( charges) == 1:
        charge = charges[0]
      else:
        charge = ''
      self.atom_charge = Pmw.Counter( self.atom_page,
                                      labelpos = 'w',
                                      label_text = _('Charge'),
                                      entryfield_value = charge,
                                      entryfield_validate={ 'validator':'integer', 'min':-4, 'max':4},
                                      entry_width = 3,
                                      increment = 1,
                                      datatype = 'integer')
      self.atom_charge.pack( anchor='nw', padx=10, pady=5)
      # show?
      shows = misc.filter_unique( [o.show for o in items if o.object_type == 'atom'])
      if len( shows) == 1:
        show = shows[0]
      else:
        show = 2 # means the show should be preserved as is
      self.atom_show = Pmw.OptionMenu( self.atom_page,
                                       labelpos = 'nw',
                                       label_text = _('Atom name'),
                                       items = (_("don't show"),_("show"),u""),
                                       initialitem = show)
      self.atom_show.pack( anchor = 'nw')
      # positioning
      poss = misc.filter_unique( [o.pos for o in items if o.object_type == 'atom'])
      if not poss:
        pos = None
      elif len( poss) == 1 and poss[0]:
        pos = ['center-first', 'center-last'].index( poss[0])
      else:
        pos = 2 # means the centering should be preserved as is
      if pos == None:
        self.atom_pos = None
      else:
        self.atom_pos = Pmw.OptionMenu( self.atom_page,
                                        labelpos = 'nw',
                                        label_text = _('Atom positioning'),
                                        items = (_("center first letter"),_("center last letter"), u""),
                                        initialitem = pos)

        self.atom_pos.pack( anchor = 'nw')
      # show hydrogens
      shows = misc.filter_unique( [o.show_hydrogens for o in items if o.object_type == 'atom'])
      if len( shows) == 1:
        show = shows[0]
      else:
        show = 2 # means the show should be preserved as is
      self.atom_show_h = Pmw.OptionMenu( self.atom_page,
                                         labelpos = 'nw',
                                         label_text = _('Hydrogens'),
                                         items = (_("off"),_("on"), u""),
                                         initialitem = pos)

      self.atom_show_h.pack( anchor = 'nw')

    # BOND
    if 'bond' in types:
      self.bond_page = self.pages.add(_('Bond'))
      # distance
      dists = misc.filter_unique( map( abs, [o.distance for o in items if o.object_type == 'bond']))
      if len( dists) == 1:
        dist = dists[0]
      else:
        dist = ''
      self.bond_dist = Pmw.Counter( self.bond_page,
                                    labelpos = 'w',
                                    label_text = _('Bond width'),
                                    entryfield_value = dist,
                                    entryfield_validate={ 'validator':'real', 'min':1, 'max':100},
                                    entry_width = 5,
                                    entryfield_modifiedcommand = self._bond_dist_changed,
                                    increment = 1,
                                    datatype = 'real')
      self.bond_dist.pack( anchor='nw', padx=10, pady=5)

    # ARROW
    if 'arrow' in types:
      self.arrow_page = self.pages.add(_('Arrow'))
      self.arrow_end_changed = 0
      self.arrow_start_changed = 0
      arrow_items = [o for o in items if o.object_type == 'arrow']

      # arrow start pins
      arrow_starts = misc.filter_unique( [o.get_pins()[0] for o in arrow_items])
      self.arrow_start = Tkinter.IntVar()
      if len( arrow_starts) == 1:
        self.arrow_start.set( arrow_starts[0])
      else:
        self.arrow_start.set( 0)
      self.arrow_start_entry = Tkinter.Checkbutton( self.arrow_page,
                                                    text=_('Arrow-head on start'),
                                                    variable = self.arrow_start,
                                                    command = self._arrow_start_changed)
      self.arrow_start_entry.pack()

      # arrow end pins
      arrow_ends = misc.filter_unique( [o.get_pins()[1] for o in arrow_items])
      self.arrow_end = Tkinter.IntVar()
      if len( arrow_ends) == 1:
        self.arrow_end.set( arrow_ends[0])
      else:
        self.arrow_end.set( 0)
      self.arrow_end_entry = Tkinter.Checkbutton( self.arrow_page,
                                                  text=_('Arrow-head on end'),
                                                  variable = self.arrow_end,
                                                  command = self._arrow_end_changed)
      self.arrow_end_entry.pack()
      

    # TEXTS

    # PLUS

    # FONT
    if ('atom' in types) or ('text' in types) or ('plus' in types):
      self.font_page = self.pages.add(_('Font'))

      font_items = [o for o in items if o.object_type == 'atom' or o.object_type == 'text' or o.object_type == 'plus']
      sizes = misc.filter_unique( [o.font_size for o in font_items])
      if len( sizes) == 1:
        size = sizes[0]
      else:
        size = ''
      self.font_size = Pmw.Counter( self.font_page,
                                    labelpos = 'nw',
                                    label_text = _('Font size'),
                                    entryfield_value = size,
                                    entryfield_validate={ 'validator': font_size_validator},
                                    entry_width = 3,
                                    increment = 1,
                                    datatype = {'counter': font_size_counter})
      self.font_size.pack( anchor = 'nw')

      used_families = misc.filter_unique( [o.font_family for o in font_items])
      available_families = list( tkFont.families())
      available_families.sort()
      available_families.insert(-1, '')
      for fnt in data.always_available_fonts:
        available_families.insert(1, fnt)
      if len( used_families) == 1:
        self.used_family = used_families[0]
        if self.used_family not in available_families:
          self.used_family = ''
      else:
        self.used_family = ''
      self.font_family = Pmw.ScrolledListBox( self.font_page,
                                              labelpos = 'nw',
                                              label_text = _('Font family'),
                                              listbox_selectmode = 'single',
                                              items = available_families,
                                              hull_relief = 'ridge')
      self.font_family.select_set( available_families.index( self.used_family))
      self.font_family.see( available_families.index( self.used_family))
      self.font_family.pack( anchor="nw", side = 'bottom')


    # COMMON
    self.common_page = self.pages.add(_('Common'))
    there_are_lines = 0
    for line_type in data.line_types:
      if line_type in types:
        there_are_lines = 1
        break
    if there_are_lines:
      line_items = [o for o in items if o.object_type in data.line_types]
      widths = misc.filter_unique( [o.width for o in line_items])
      if len( widths) == 1:
        width = widths[0]
      else:
        width = ''
      self.line_width = Pmw.Counter( self.common_page,
                                      labelpos = 'w',
                                      label_text = _('Line width'),
                                      entryfield_value = width,
                                      entryfield_validate={ 'validator':'real', 'min':0.1, 'max':20},
                                      entry_width = 4,
                                      increment = 0.2,
                                      datatype = 'real')
      self.line_width.pack( anchor='nw', padx=10, pady=5)

    fill_color_items = [o for o in items if o.object_type in data.fill_color_types]
    fills = misc.filter_unique( [o.fill for o in fill_color_items])
    if len( fills) == 1:
      fill = fills[0]
    else:
      fill = None
    self.fill_color = widgets.ColorButton( self.common_page, color=fill, text=_("Fill color"))
    self.fill_color.pack( anchor='nw', padx=10, pady=5)

    outline_color_items = [o for o in items if o.object_type in data.outline_color_types]
    outlines = misc.filter_unique( [o.outline for o in outline_color_items])
    if len( outlines) == 1:
      outline = outlines[0]
    else:
      outline = None
    self.outline_color = widgets.ColorButton( self.common_page, color=outline, text=_("Outline color"))
    self.outline_color.pack( anchor='nw', padx=10, pady=5)


    # RUN IT ALL
    self.pages.setnaturalsize()
    self.dialog.activate( globalMode=0)


  def done( self, button):
    """called on dialog exit"""
    self.dialog.deactivate()
    if button != _('OK'):
      pass
    else:
      # apply changes
      for o in self.items:
        change = 0
        # ATOM
        if o.object_type == 'atom':
          a = self.atom_show.index( Pmw.SELECT)
          if a != 2:
            o.show = a
            change = 1
          # positionning
          a = self.atom_pos.index( Pmw.SELECT)
          if a != 2:
            o.pos = ('center-first', 'center-last')[ a]
            change = 1
          if self.atom_charge.get():
            a = int( self.atom_charge.get())
            if o.charge != a:
              o.charge = a
            change = 1
          # hydrogens
          a = int( self.atom_show_h.index( Pmw.SELECT))
          if a != 2:
            o.show_hydrogens = a
            change = 1
          # font is in common now
        # BOND
        elif o.object_type == 'bond':
          # width is in common now
          # distance
          if self.bond_dist.get():
            d = float( self.bond_dist.get())
            if d != abs( o.distance):
              o.distance = d * misc.signum( o.distance)
              change = 1
        # ARROW - most is in common now
        elif o.object_type == 'arrow':
          if self.arrow_start_changed:
            o.set_pins( start = self.arrow_start.get())
            change = 1
          if self.arrow_end_changed:
            o.set_pins( end = self.arrow_end.get())
            change = 1

        # TEXT - all is in common now
        # PLUS - all is in common now
        # VECTOR - all is in common now

        # COMMON PROPERTIES
        # FILL COLOR
        if o.object_type in data.fill_color_types:
          if self.fill_color.color:
            if self.fill_color.color != o.fill:
              o.fill = self.fill_color.color
              change = 1
        # OUTLINE COLOR
        if o.object_type in data.outline_color_types:
          if self.outline_color.color:
            if self.outline_color.color != o.outline:
              o.outline = self.outline_color.color
              change = 1
        # LINE WIDTH
        if o.object_type in data.line_types:
          if self.line_width.get():
            w = float( self.line_width.get())
            if w != o.width:
              o.width = w
              change = 1
        # FONT
        if o.object_type in data.font_types:
          if self.font_size.get():
            a = int( self.font_size.get())
            o.font_size = a
            change = 1
          if self.font_family.getcurselection() and self.font_family.getcurselection()[0] != self.used_family:
            a = self.font_family.getcurselection()[0]
            o.font_family = a
            change = 1
          
        # APPLY THE CHANGES
        if change:
          o.redraw()
          self.changes_made = 1
    self.cleanup()

  def _bond_width_changed( self):
    pass

  def _bond_dist_changed( self):
    pass

  def _arrow_end_changed( self):
    self.arrow_end_changed = 1

  def _arrow_start_changed( self):
    self.arrow_start_changed = 1

  def raise_me( self, event):
    self.dialog.tkraise()

  def cleanup( self):
    pass
    #self.parent.unbind_all( "<Button-1>")

def font_size_validator( input):
  if input == '':
    return Pmw.OK
  if len( input) > 3:
    return Pmw.ERROR
  if re.match( "^\d+$", input):
    i = int( input)
    if i >= 1 and i <= 64:
      return Pmw.OK
    else:
      return Pmw.PARTIAL
  return Pmw.ERROR


def font_size_counter( text, factor, increment):
  if text == '':
    return 12
  i = int( text)
  i += factor * increment
  return i
