#--------------------------------------------------------------------------
#     This file is part of BKchem - a chemical drawing program
#     Copyright (C) 2002  Beda Kosata <kosatab@vscht.cz>

#     This program is free software; you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation; either version 2 of the License, or
#     (at your option) any later version.

#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.

#     Complete text of GNU GPL can be found in the file gpl.txt in the
#     main directory of the program

#--------------------------------------------------------------------------
#
# Last edited: $Date: 2003/03/23 21:22:14 $
#
#--------------------------------------------------------------------------

"""some extensions to DOM for more convenient work"""

import re
import xml.dom.minidom as dom
import operator
#from __future__ import division

class XMLPrettyPrinter:
  """prints XML in a very nice fashion :)
  has modes - text - for more textish documents - like HTML
            - data - for documents with more tags then text"""

  def __init__( self, xml_in_text, mode = 'text'):
    if mode not in ('text','data'):
      raise "bad mode for XMLPrettyPrinter"
    self.mode = mode
    self.text = self.nice_format_data( xml_in_text)

  def get( self):
    return self.text

  def nice_format_data( self, a):
    a = re.sub( "'", "\"", a)
    self._indent = 0
    a = re.sub( "(\/|\?)?(?!\?)(>)([^<>]*)(<)(\/)?", self.indent, a)
    return a

  def indent( self, match):
    indent = self._indent
    if not match.group( 1):
      i1 = ''
      indent = indent + 2
    else: i1 = match.group( 1)
    
    if match.group( 5):
      indent = indent - 2
      i5 = match.group( 5)
    else: i5 = ''
      
    if match.group( 3):
      if self.mode == 'text':
        i3 = '  '+match.group( 3)+'\n'+indent*' '
      else:
        i3 = match.group( 3)
    else: i3 = ''
    i = indent*' ' #(indent / 4)*'\t'+(indent % 4)*' '
    if match.group( 5): indent = indent -2 # remember the endtag for the next run
    self._indent = indent
    if self.mode == 'text':
      return i1+">\n"+i+i3+'<'+i5
    else:
      if i3:
        return i1+'>'+i3+'<'+i5
      else:
        return i1+'>\n'+i+i3+'<'+i5

def safe_indent( element, level=0, step=2):
  """indents DOM tree. Does not add any extra whitespaces to text elements."""
  if not element.childNodes:
    pass
  elif reduce( operator.add, [isinstance( child, dom.Text) for child in element.childNodes], 0):
    pass
  elif element.nodeName == 'ftext' or element.nodeName == 'text':
    pass
  else:
    for child in element.childNodes[:]:
      element.insertBefore( dom.Text("\n"+(level+step)*" "), child)
      safe_indent( child, level=level+step, step=step)
    element.appendChild(  dom.Text("\n"+level*" "))


def elementUnder( parent, name, attributes=()):
  """creates element inside of parent and returns it,
  attributes are as sequence of (name,value) sequences"""
  a = dom.Element( name)
  parent.appendChild( a)
  if attributes:
    for i in attributes:
      a.setAttribute( i[0], i[1])
  return a

def textOnlyElement( name, text, attributes=()):
    a = dom.Element( name)
    a.appendChild( dom.Text( text))
    if attributes:
      for i in attributes:
        a.setAttribute( i[0], i[1])
    return a

def textOnlyElementUnder( parent, name, text, attributes=()):
  a = dom.Element( name)
  if attributes:
    for i in attributes:
      a.setAttribute( i[0], i[1])
  a.appendChild( dom.Text( text))
  parent.appendChild( a)
  return a
  
def getTextFromElement( element):
  text = ''
  for child in element.childNodes:
    if child.nodeValue:
      text += child.nodeValue
  return text

def childNodesWithoutEmptySpaces( node):
  return filter( isNotEmptyText, node.childNodes)

def isNotEmptyText( element):
  empty = re.compile('^\s*$')
  if element.nodeValue and empty.match( element.nodeValue): #(element.nodeValue == '\n') or (element.nodeValue == '\t'):
    return 0
  else:
    return 1

def getAllTextFromElement( element):
  """like getTextFromElement but mines text also from child nodes"""
  text = ''
  for child in element.childNodes:
    if child.nodeValue:
      text += child.nodeValue
    else:
      text += getAllTextFromElement( child)
  return text

def setAttributes( element, attributes):
  for a in attributes:
    element.setAttribute( a[0], a[1])
  
def getAttributes( element, names):
  """returns a list of attribute values from a list of attr names"""
  return [element.getAttribute( n) for n in names]

def getParentNameList( element):
  """returns a list of parent names (from father to grandfather...)"""
  par = element.parentNode
  if par != None:
    return [par.nodeName] + getParentNameList( par)
  else:
    return []


